@@ -143,7 +143,7 @@ class Root:
         # print 'Finish:', self.depth
         
 class Port(utils.core.Observer):
-    def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
+    def update(self,event,*args,**kw): # 監視しているストレージが更新したら直接起動されるメソッド。
         if event == EVENT_GET: return # もしGETイベントならばなにもしない
         callbacks = list(self._callbacks)
         for callback in callbacks:
@@ -153,7 +153,7 @@ class Port(utils.core.Observer):
             callback.__call__() # コールバックを実行し
     def __init__(self,*args,**kw):
         if utils.isfamily(args[0],Port): # 引数がポート一族であれば
-            s = Storage(args[0].storage.value) # ポートのデータを初期値として新規サブジェクトを生成し
+            s = Storage(args[0].subject.value) # ポートのデータを初期値として新規サブジェクトを生成し
             utils.core.Observer.__init__(self,s) # サブジェクトを監視するオブザーバを生成する
         else: # 引数がその他であれば
             s = Storage(*args,**kw) # 引数を初期値としてサブジェクトを生成し
@@ -246,8 +246,8 @@ class Port(utils.core.Observer):
         #print 'Fixed:', Root().fixed
         schedule.queue, schedule.ignore, schedule.fixed = escape
         del self._setnow_called
-    def get(self):
-        return self.subject.get()
+    def get(self,*args,**kw):
+        return self.subject.get(*args,**kw)
         
 class Parm(Port): # 幅優先実行のポート
     def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
@@ -266,7 +266,7 @@ class Parm(Port): # 幅優先実行のポート
                 callback.__call__() # コールバックを実行する
         
 class Output(Port): # PULL型のポート
-    def update(self,event,*args): # 監視しているストレージが更新したら直接起動されるメソッド。
+    def update(self,event,*args,**kw): # 監視しているストレージが更新したら直接起動されるメソッド。
         if event == EVENT_SET: return # もしSETイベントならばなにもしない
         schedule = Scheduler().get(threading.currentThread())
         if schedule.fixed.count(self.subject): return # もし確定済みならばなにもしない
@@ -278,7 +278,7 @@ class Output(Port): # PULL型のポート
             if not schedule.ignore.count(callback): # 最初の実行ならば
                 if Scheduler().trace: self.print_callback(callback)
                 schedule.ignore.append(callback) # 実行フラグを立てて
-                callback.__call__() # コールバックを実行し
+                callback.__call__(*args,**kw) # コールバックを実行し
                 # print 'Ignored:', Root().ignore.count(callback), cwl(Root().ignore)
         if Scheduler().trace: self.print_queue()
     def print_callback(self,callback):
@@ -342,9 +342,9 @@ class Storage(utils.core.Subject):
             schedule.fixed  = []
         else:
             self.notify(EVENT_SET)
-    def get(self):
+    def get(self,*args,**kw):
         '''格納しているオブジェクトまたはリテラルを返す'''
-        self.notify(EVENT_GET)
+        self.notify(EVENT_GET,*args,**kw)
         return self.value
 
 class Component:
