@@ -26,10 +26,12 @@ class Polling(Tkinter.Frame):
         self._id      = None
         self._polling()
     def start(self):
-        self._polling()
+        try: self._id
+        except AttributeError: self._polling()
     def stop(self):
-        if not self._id == None:
-            self.after_cancel(self._id)
+        try: self.after_cancel(self._id)
+        except: return
+        del self._id
     def _polling(self):
         self.value.set_now(None)
         self._id = self.after(self.interval,self._polling)
